/* 
 * Un oggetto Punto rappresenta un punto
 * in uno spazio bidimensionale. 
 */
public class Punto {
    /* Coordinata x di questo punto. */
    private double x;
    /* Coordinata y di questo punto. */
    private double y;

    /* Crea un nuovo Punto di coordinate (x,y). */
    public Punto(double x, double y) {
        this.x = x;
        this.y = y;
    }

    /* Crea un nuovo Punto di coordinate (0,0). */
    public Punto() {
        this.x = 0;
        this.y = 0;
    }

    /* Crea un nuovo Punto uguale al Punto p. */
    public Punto(Punto p) {
        // pre: p!=null
        this.x = p.x;
        this.y = p.y;
    }

    /* Restituisce la coordinata x di questo punto. */
    public double getX() {
        return this.x;
    }
    
    /* Restituisce la coordinata y di questo punto. */
    public double getY() {
        return this.y;
    }
    
    /* Trasla questo punto di dx in orizzontale
     * e di dy in verticale. */
    public void trasla(double dx, double dy) {
        this.x = this.x + dx;
        this.y = this.y + dy;
    }
    
    /* Calcola una descrizione testuale di questo punto. */
    public String toString() {
        return "Punto[x=" + x + ",y=" + y + "]";
    }

    /* Verifica se questo punto  uguale al Punto p. */
    public boolean equals(Punto p) {
        if (p==null) {
        	return false; 
        } else {
            return (this.x==p.x) && (this.y==p.y);        	
        }
    }

    /* Verifica se questo punto  uguale all'oggetto o. 
     * Vedi capitolo 25. */
    public boolean equals(Object o) {
        if (o==null) {
        	return false; 
        } else if (this.getClass()!=o.getClass()) {
        	return false; 
        } else {
        	/* anche o  un Punto */ 
        	Punto p = (Punto) o; 
            return (this.x==p.x) && (this.y==p.y);        	
        }
    }

    /* Calcola la distanza tra gli oggetti Punto p1 e p2. */
    public static double distanza(Punto p1, Punto p2) {
        double qd;    // quadrato della distanza tra p1 e p2

        /* calcola il quadrato della distanza tra p1 e p2 */
        qd = (p1.x-p2.x)*(p1.x-p2.x) + (p1.y-p2.y)*(p1.y-p2.y);
        /* calcola e restituisce la distanza tra p1 e p2 */
        return Math.sqrt(qd);
    }
}
