/* 
 * Un oggetto Rettangolo rappresenta un rettangolo in uno spazio bidimensionale, 
 * i cui lati sono entrambi paralleli agli assi cartesiani del piano. 
 */
public class Rettangolo {
    /* Vertice in basso a sinistra di questo rettangolo. */
    private Punto vbs;
    /* Vertice in alto a destra di questo rettangolo. */
    private Punto vad;

    /* Crea un nuovo Rettangolo tra i punti p1 e p2. */
    public Rettangolo(Punto p1, Punto p2) {
        // pre: p1!=null && p2!=null
        this.vbs = new Punto(Math.min(p1.getX(),p2.getX()),
                             Math.min(p1.getY(),p2.getY()));
        this.vad = new Punto(Math.max(p1.getX(),p2.getX()),
                             Math.max(p1.getY(),p2.getY()));
    }
    
    /* Crea un nuovo Rettangolo che  inizialmente uguale
     * al Rettangolo r. */
    public Rettangolo(Rettangolo r) {
        // pre: r!=null
        this.vbs = new Punto(r.vbs);
        this.vad = new Punto(r.vad);
    }

    /* Calcola la base di questo rettangolo. */
    public double base() {
        return vad.getX() - vbs.getX();
    }
    
    /* Calcola l'altezza di questo rettangolo. */
    public double altezza() {
        return vad.getY() - vbs.getY();
    }
    
    /* Calcola l'area di questo rettangolo. */
    public double area() {
        return this.base()*this.altezza();
    }
    
    /* Trasla questo rettangolo di dx in orizzontale
     * e di dy in verticale. */
    public void trasla(double dx, double dy) {
        vbs.trasla(dx,dy);
        vad.trasla(dx,dy);
    }
    
    /* Verifica se questo rettangolo  uguale al Rettangolo r. */
    public boolean equals(Rettangolo r) {
        // pre: r!=null
        return (this.vbs.equals(r.vbs)) &&
               (this.vad.equals(r.vad));
    }
    
    /* Verifica se questo rettangolo  uguale all'oggetto o. */
    public boolean equals(Object o) {
    	if (o==null) {
    		return false; 
    	} else if (this.getClass()!=o.getClass()) {
    		return false; 
    	} else {
    		Rettangolo r = (Rettangolo) o; 
            return (this.vbs.equals(r.vbs)) &&
                    (this.vad.equals(r.vad));
    	}
    }

    /* Restituisce una descrizione di questo rettangolo. */
    public String toString() {
        return "Rettangolo[vbs=" + vbs.toString() +
               ",vad=" + vad.toString() + "]";
    }
    
    /* Calcola il rettangolo intersezione tra r1 e r2.
     * Restituisce null se l'intersezione  vuota. */
    public static Rettangolo intersezione(Rettangolo r1,
                                          Rettangolo r2) {
        // pre: r1!=null && r2!=null
        Rettangolo inters;    // intersezione di r1 e r2
        if (r1.vbs.getX()>r2.vad.getX() ||
            r1.vbs.getY()>r2.vad.getY() ||
            r1.vad.getX()<r2.vbs.getX() ||
            r1.vad.getY()<r2.vbs.getY())
                inters = null;
        else
            inters =
              new Rettangolo(
                new Punto(Math.max(r1.vbs.getX(),r2.vbs.getX()),
                          Math.max(r1.vbs.getY(),r2.vbs.getY())),
                new Punto(Math.min(r1.vad.getX(),r2.vad.getX()),
                          Math.min(r1.vad.getY(),r2.vad.getY())));
        return inters;
    }
}
