/* 
 * Calcolo ricorsivo del numero di isole in un arcipelago. 
 */
public class Isole {

    /* Calcola il numero di isole nell'arcipelago 
     * rappresentato dalla mappa m.
     * La mappa m  un array bidimensionale che
     * memorizza una bitmap che descrive un arcipelago.
     * Ogni elementi di a rappresenta un'area quadrata:
     * se vale 0 vuol dire che c' il mare,
     * se invece vale 1 vuol dire che c' terra.
     * Le isole sono dunque rappresentate da gruppi di 1 adiacenti.
     * Ipotizziamo che le isole non si toccano. */
    public static int isole(int[][] m) {
        // pre: m!=null && ogni riga di m  non nulla
        int i, j;    // indici per la scansione di m
        int isole;    // numero di isole sulla mappa
        int[][] c;    // un clone della mappa m

        /* l'idea  contare le isole come segue:
         * ogni volta che incontra un 1, conta una nuova isola
         * e cancella l'isola */

        /* lavora su un clone per non modificare m */
        c = clona(m);

        /* scandisci la mappa e conta le isole */ 
        isole = 0;
        for (i=0; i<c.length; i++) {
            for (j=0; j<c[i].length; j++) {
                if (c[i][j]==1) {
					isole++;
					cancellaIsola(c,i,j);
				}
            }
        }
		return isole;
    }

    /* Crea una copia dell'array di array a. */
    public static int[][] clona(int[][] a) {
        // pre: a!=null && le righe di a sono non nulle
        int[][] c;    // array di array clone di a
        int i;    // indice per la scansione delle righe di a e c

        /* crea c, della stessa lunghezza di a */
        c = new int[a.length][];
        /* copia i cloni degli elementi di a in c */
        for (i=0; i<a.length; i++) {
            c[i] = clona(a[i]);
        }
        /* restituisce il clone */
        return c;
    }

    /* Crea una copia dell'array a. */
    public static int[] clona(int[] a) {
        // pre: a!=null
        int[] c;    // array clone di a
        int i;      // indice per la scansione di a e c

        /* crea c, della stessa lunghezza di a */
        c = new int[a.length];
        /* copia gli elementi di a in c */
        for (i=0; i<a.length; i++) {
            c[i] = a[i];
        }
        /* restituisce il clone */
        return c;
    }

    /* Cancella l'isola che occupa la posizione i,j
     * dalla mappa m */
    public static void cancellaIsola(int[][] m, int i, int j) {
        // pre: m!=null && ogni riga di m  non nulla

        /* cancella l'isola in modo ricorsivo */
        if (i>=0 && i<m.length && j>=0 && j<m[i].length && m[i][j]==1) {
			/* cancella la terra nella posizione i,j */
			m[i][j] = 0;
			/* cancella ricorsivamente l'isola 
			 * dalle posizioni intorno a i,j */
			cancellaIsola(m,i-1,j-1);
			cancellaIsola(m,i-1,j);
			cancellaIsola(m,i-1,j+1);
			cancellaIsola(m,i,j-1);
			cancellaIsola(m,i,j+1);
			cancellaIsola(m,i+1,j-1);
			cancellaIsola(m,i+1,j);
			cancellaIsola(m,i+1,j+1);
		}
    }

}