/* 
 * Ordinamento per fusione. 
 */ 
public class MergeSort { 

    /* Ordinamento l'array dati in modo non decrescente.
     * Ordinamento per fusione. */
    public static void mergeSort(int[] dati) {
        // pre: dati!=null
        int n;            // lunghezza di dati

        /* ordina dati in modo non decrescente */
        n = dati.length;
        /* avvia la ricorsione:
         * ordina dati, dall'elemento di posizione 0 (compreso)
         * a quello di posizione n (escluso)
         * usando un array di appoggio di lunghezza n */
        mergeSortRic(dati, 0, n, new int[n]);
    }

    /* Ordina gli elementi di dati tra sinistra (incluso) e
     * destra (escluso) usando l'array di appoggio temp. */
    private static void mergeSortRic(int[] dati,
            int sinistra, int destra, int[] temp) {
        int m;         // numero di elementi da ordinare
        int centro;    // indice dell'elemento centrale
                       // della sottosequenza da ordinare

        /* ordina gli elementi di dati tra sinistra
         * (incluso) e destra (escluso) */
        m = destra-sinistra;
        if (m==2) {          // caso base, confronto e scambio
            if (dati[sinistra]>dati[destra-1])
                scambia(dati, sinistra, destra-1);
        } else if (m>2) {    // caso induttivo
            centro = (sinistra+destra)/2;
            mergeSortRic(dati, sinistra, centro, temp);
            mergeSortRic(dati, centro, destra, temp);
            merge(dati, sinistra, centro, destra, temp);
        }
        /* m<2  un altro caso base: la sottosequenza  ordinata */
    }

    /* Fonde le sotto-sequenze contigue e ordinate
     * (sinistra, centro) e (centro, destra) di dati,
     * usando l'array di appoggio temp. */
    private static void merge(int[] dati,
            int sinistra, int centro, int destra,
            int[] temp) {
        // pre: le seguenti sotto-sequenze di dati
        //      sono ordinate in modo non decrescente:
        //      da sinistra (incluso) a centro (escluso)
        //      da centro (incluso) a destra (escluso)
        int i1, i2, i;       // indici per le sotto-sequenze

        /* inizializzazioni */
        i1 = sinistra;    i2 = centro;    i = sinistra;
        /* fusione: prima fase */
        while (i1<centro && i2<destra) {
            /* seleziona il pi piccolo tra dati[i1] e dati[i2] */
            if (dati[i1]<dati[i2]) {
                temp[i] = dati[i1];
                i1++;
            } else {  // allora dati[i1]>=dati[i2]
                temp[i] = dati[i2];
                i2++;
            }
            i++;
        }
        /* ora i1>=centro || i2>=destra */
        /* fusione: seconda fase */
        while (i1<centro) {
            temp[i] = dati[i1];
            i1++;    i++;
        }
        while (i2<destra) {
            temp[i] = dati[i2];
            i2++;    i++;
        }
        /* ora copia da temp a dati */
        for (i=sinistra; i<destra; i++) {
            dati[i] = temp[i];
        }
    }
    
    /* Scambia gli elementi di indice i e j di dati. */
    private static void scambia(int[] dati, int i, int j) {
        // pre: dati!=null && 0<= i,j <dati.length
        int temp;    // variabile di supporto per lo scambio

        /* scambia dati[i] con dati[j] */
        temp = dati[i];
        dati[i] = dati[j];
        dati[j] = temp;
    }

} 